"""Humanoid benchmark tasks."""

# ruff: noqa: F401

from .balance_metacfg import BalanceMetaCfg
from .base_metacfg import HumanoidTaskMetaCfg
from .crawl_metacfg import CrawlMetaCfg
from .cube_metacfg import CubeMetaCfg
from .door_metacfg import DoorMetaCfg
from .hurdle_metacfg import HurdleMetaCfg
from .maze_metacfg import MazeMetaCfg
from .package_metacfg import PackageMetaCfg
from .pole_metacfg import PoleMetaCfg
from .powerlift_metacfg import PowerliftMetaCfg
from .push_metacfg import PushMetaCfg
from .run_metacfg import RunMetaCfg
from .sit_metacfg import SitMetaCfg
from .slide_metacfg import SlideMetaCfg
from .spoon_metacfg import SpoonMetaCfg
from .stair_metacfg import StairMetaCfg
from .stand_metacfg import StandMetaCfg
from .walk_metacfg import WalkMetaCfg
